import { ElemType, Eq, IfElse, RequireString, Simplify } from './utils';
import { Adapt, Apply, Choice, Compose, DisallowTypeChange, ElemUnion, Elems, HKT, Id, Path2, Path3, Path4, Path5, Plant, Prop, Optional, Union } from './hkt';
export { Apply, Compose, Eq, HKT };
export declare type Removable = true | undefined;
export interface Params<T extends HKT, R extends Removable = undefined> {
    readonly _T: T;
    readonly _R: R;
}
export declare type OpticParams = Params<any, any>;
export declare type NextParams<C extends OpticParams, T extends HKT, R extends Removable = undefined> = Params<Compose<C['_T'], T>, R>;
export declare type NextComposeParams<C1 extends OpticParams, C2 extends OpticParams> = Params<Compose<C1['_T'], C2['_T']>, C2['_R']>;
export declare type OpticFor<S> = Equivalence<S, Params<DisallowTypeChange<S>>, S>;
export declare type OpticFor_<S> = Equivalence<S, Params<Id>, S>;
export interface Equivalence<S, T extends OpticParams, A> {
    readonly _tag: 'Equivalence';
    readonly _removable: T['_R'];
    compose<T2 extends OpticParams, A2>(optic: Equivalence<A, T2, A2>): Equivalence<S, NextComposeParams<T, T2>, A2>;
    compose<T2 extends OpticParams, A2>(optic: Iso<A, T2, A2>): Iso<S, NextComposeParams<T, T2>, A2>;
    iso<U>(there: (a: A) => U, back: (u: U) => A): Iso<S, NextParams<T, Adapt<A, U>>, U>;
    compose<T2 extends OpticParams, A2>(optic: Lens<A, T2, A2>): Lens<S, NextComposeParams<T, T2>, A2>;
    prop<K extends keyof A>(key: K): Lens<S, NextParams<T, Prop<A, K>>, A[K]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2], K4 extends keyof A[K1][K2][K3], K5 extends keyof A[K1][K2][K3][K4]>(path: [K1, K2, K3, K4, K5]): Lens<S, NextParams<T, Path5<A, K1, K2, K3, K4, K5>>, A[K1][K2][K3][K4][K5]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2], K4 extends keyof A[K1][K2][K3]>(path: [K1, K2, K3, K4]): Lens<S, NextParams<T, Path4<A, K1, K2, K3, K4>>, A[K1][K2][K3][K4]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2]>(path: [K1, K2, K3]): Lens<S, NextParams<T, Path3<A, K1, K2, K3>>, A[K1][K2][K3]>;
    path<K1 extends keyof A, K2 extends keyof A[K1]>(path: [K1, K2]): Lens<S, NextParams<T, Path2<A, K1, K2>>, A[K1][K2]>;
    path<K1 extends keyof A>(path: [K1]): Lens<S, NextParams<T, Prop<A, K1>>, A[K1]>;
    pick<K extends keyof A>(keys: K[]): Lens<S, NextParams<T, Plant<A, K>>, Pick<A, K>>;
    filter(predicate: (item: ElemType<A>) => boolean): Lens<S, NextParams<T, Union<A>>, A>;
    valueOr<B>(defaultValue: B): Lens<S, NextParams<T, Id>, Exclude<A, undefined> | B>;
    compose<T2 extends OpticParams, A2>(optic: Prism<A, T2, A2>): Prism<S, NextComposeParams<T, T2>, A2>;
    optional(): Prism<S, NextParams<T, Optional>, Exclude<A, undefined>>;
    guard_<F extends HKT>(): <U extends A>(g: (a: A) => a is U) => Prism<S, NextParams<T, F>, U>;
    guard<U extends A>(g: (a: A) => a is U): Prism<S, NextParams<T, Choice<A, U>>, U>;
    find(predicate: (item: ElemType<A>) => boolean): Prism<S, NextParams<T, ElemUnion<A>, true>, ElemType<A>>;
    when(predicate: (item: A) => boolean): Prism<S, NextParams<T, Union<A>>, A>;
    at(i: number): IfElse<Eq<A, string>, Prism<S, NextParams<T, DisallowTypeChange<string>, true>, string>, Prism<S, NextParams<T, ElemUnion<A>, true>, ElemType<A>>>;
    head(): IfElse<Eq<A, string>, Prism<S, NextParams<T, DisallowTypeChange<string>, true>, string>, Prism<S, NextParams<T, ElemUnion<A>, true>, ElemType<A>>>;
    index(i: number): IfElse<Eq<A, string>, Prism<S, NextParams<T, DisallowTypeChange<string>, true>, string>, Prism<S, NextParams<T, ElemUnion<A>, true>, ElemType<A>>>;
    compose<T2 extends OpticParams, A2>(optic: Traversal<A, T2, A2>): Traversal<S, NextComposeParams<T, T2>, A2>;
    elems(): Traversal<S, NextParams<T, Elems>, ElemType<A>>;
    chars(): RequireString<A, Traversal<S, NextParams<T, DisallowTypeChange<string>>, string>>;
    words(): RequireString<A, Traversal<S, NextParams<T, DisallowTypeChange<string>>, string>>;
    compose<A2>(optic: Getter<A, A2>): Getter<S, A2>;
    to<B>(f: (a: A) => B): Getter<S, B>;
    compose<A2>(optic: AffineFold<A, A2>): AffineFold<S, A2>;
    compose<A2>(optic: Fold<A, A2>): Fold<S, A2>;
    compose<T2 extends OpticParams, A2>(optic: Setter<A, T2, A2>): Setter<S, NextComposeParams<T, T2>, A2>;
    prependTo(): Setter<S, NextParams<T, ElemUnion<A>>, ElemType<A>>;
    appendTo(): Setter<S, NextParams<T, ElemUnion<A>>, ElemType<A>>;
}
export interface Iso<S, T extends OpticParams, A> {
    readonly _tag: 'Iso';
    readonly _removable: T['_R'];
    compose<T2 extends OpticParams, A2>(optic: Equivalence<A, T2, A2>): Iso<S, NextComposeParams<T, T2>, A2>;
    compose<T2 extends OpticParams, A2>(optic: Iso<A, T2, A2>): Iso<S, NextComposeParams<T, T2>, A2>;
    iso<U>(there: (a: A) => U, back: (u: U) => A): Iso<S, NextParams<T, Adapt<A, U>>, U>;
    compose<T2 extends OpticParams, A2>(optic: Lens<A, T2, A2>): Lens<S, NextComposeParams<T, T2>, A2>;
    prop<K extends keyof A>(key: K): Lens<S, NextParams<T, Prop<A, K>>, A[K]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2], K4 extends keyof A[K1][K2][K3], K5 extends keyof A[K1][K2][K3][K4]>(path: [K1, K2, K3, K4, K5]): Lens<S, NextParams<T, Path5<A, K1, K2, K3, K4, K5>>, A[K1][K2][K3][K4][K5]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2], K4 extends keyof A[K1][K2][K3]>(path: [K1, K2, K3, K4]): Lens<S, NextParams<T, Path4<A, K1, K2, K3, K4>>, A[K1][K2][K3][K4]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2]>(path: [K1, K2, K3]): Lens<S, NextParams<T, Path3<A, K1, K2, K3>>, A[K1][K2][K3]>;
    path<K1 extends keyof A, K2 extends keyof A[K1]>(path: [K1, K2]): Lens<S, NextParams<T, Path2<A, K1, K2>>, A[K1][K2]>;
    path<K1 extends keyof A>(path: [K1]): Lens<S, NextParams<T, Prop<A, K1>>, A[K1]>;
    pick<K extends keyof A>(keys: K[]): Lens<S, NextParams<T, Plant<A, K>>, Pick<A, K>>;
    filter(predicate: (item: ElemType<A>) => boolean): Lens<S, NextParams<T, Union<A>>, A>;
    valueOr<B>(defaultValue: B): Lens<S, NextParams<T, Id>, Exclude<A, undefined> | B>;
    compose<T2 extends OpticParams, A2>(optic: Prism<A, T2, A2>): Prism<S, NextComposeParams<T, T2>, A2>;
    optional(): Prism<S, NextParams<T, Optional>, Exclude<A, undefined>>;
    guard_<F extends HKT>(): <U extends A>(g: (a: A) => a is U) => Prism<S, NextParams<T, F>, U>;
    guard<U extends A>(g: (a: A) => a is U): Prism<S, NextParams<T, Choice<A, U>>, U>;
    find(predicate: (item: ElemType<A>) => boolean): Prism<S, NextParams<T, ElemUnion<A>, true>, ElemType<A>>;
    when(predicate: (item: A) => boolean): Prism<S, NextParams<T, Union<A>>, A>;
    at(i: number): IfElse<Eq<A, string>, Prism<S, NextParams<T, DisallowTypeChange<string>, true>, string>, Prism<S, NextParams<T, ElemUnion<A>, true>, ElemType<A>>>;
    head(): IfElse<Eq<A, string>, Prism<S, NextParams<T, DisallowTypeChange<string>, true>, string>, Prism<S, NextParams<T, ElemUnion<A>, true>, ElemType<A>>>;
    index(i: number): IfElse<Eq<A, string>, Prism<S, NextParams<T, DisallowTypeChange<string>, true>, string>, Prism<S, NextParams<T, ElemUnion<A>, true>, ElemType<A>>>;
    compose<T2 extends OpticParams, A2>(optic: Traversal<A, T2, A2>): Traversal<S, NextComposeParams<T, T2>, A2>;
    elems(): Traversal<S, NextParams<T, Elems>, ElemType<A>>;
    chars(): RequireString<A, Traversal<S, NextParams<T, DisallowTypeChange<string>>, string>>;
    words(): RequireString<A, Traversal<S, NextParams<T, DisallowTypeChange<string>>, string>>;
    compose<A2>(optic: Getter<A, A2>): Getter<S, A2>;
    to<B>(f: (a: A) => B): Getter<S, B>;
    compose<A2>(optic: AffineFold<A, A2>): AffineFold<S, A2>;
    compose<A2>(optic: Fold<A, A2>): Fold<S, A2>;
    compose<T2 extends OpticParams, A2>(optic: Setter<A, T2, A2>): Setter<S, NextComposeParams<T, T2>, A2>;
    prependTo(): Setter<S, NextParams<T, ElemUnion<A>>, ElemType<A>>;
    appendTo(): Setter<S, NextParams<T, ElemUnion<A>>, ElemType<A>>;
}
export interface Lens<S, T extends OpticParams, A> {
    readonly _tag: 'Lens';
    readonly _removable: T['_R'];
    compose<T2 extends OpticParams, A2>(optic: Equivalence<A, T2, A2>): Lens<S, NextComposeParams<T, T2>, A2>;
    compose<T2 extends OpticParams, A2>(optic: Iso<A, T2, A2>): Lens<S, NextComposeParams<T, T2>, A2>;
    iso<U>(there: (a: A) => U, back: (u: U) => A): Lens<S, NextParams<T, Adapt<A, U>>, U>;
    compose<T2 extends OpticParams, A2>(optic: Lens<A, T2, A2>): Lens<S, NextComposeParams<T, T2>, A2>;
    prop<K extends keyof A>(key: K): Lens<S, NextParams<T, Prop<A, K>>, A[K]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2], K4 extends keyof A[K1][K2][K3], K5 extends keyof A[K1][K2][K3][K4]>(path: [K1, K2, K3, K4, K5]): Lens<S, NextParams<T, Path5<A, K1, K2, K3, K4, K5>>, A[K1][K2][K3][K4][K5]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2], K4 extends keyof A[K1][K2][K3]>(path: [K1, K2, K3, K4]): Lens<S, NextParams<T, Path4<A, K1, K2, K3, K4>>, A[K1][K2][K3][K4]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2]>(path: [K1, K2, K3]): Lens<S, NextParams<T, Path3<A, K1, K2, K3>>, A[K1][K2][K3]>;
    path<K1 extends keyof A, K2 extends keyof A[K1]>(path: [K1, K2]): Lens<S, NextParams<T, Path2<A, K1, K2>>, A[K1][K2]>;
    path<K1 extends keyof A>(path: [K1]): Lens<S, NextParams<T, Prop<A, K1>>, A[K1]>;
    pick<K extends keyof A>(keys: K[]): Lens<S, NextParams<T, Plant<A, K>>, Pick<A, K>>;
    filter(predicate: (item: ElemType<A>) => boolean): Lens<S, NextParams<T, Union<A>>, A>;
    valueOr<B>(defaultValue: B): Lens<S, NextParams<T, Id>, Exclude<A, undefined> | B>;
    compose<T2 extends OpticParams, A2>(optic: Prism<A, T2, A2>): Prism<S, NextComposeParams<T, T2>, A2>;
    optional(): Prism<S, NextParams<T, Optional>, Exclude<A, undefined>>;
    guard_<F extends HKT>(): <U extends A>(g: (a: A) => a is U) => Prism<S, NextParams<T, F>, U>;
    guard<U extends A>(g: (a: A) => a is U): Prism<S, NextParams<T, Choice<A, U>>, U>;
    find(predicate: (item: ElemType<A>) => boolean): Prism<S, NextParams<T, ElemUnion<A>, true>, ElemType<A>>;
    when(predicate: (item: A) => boolean): Prism<S, NextParams<T, Union<A>>, A>;
    at(i: number): IfElse<Eq<A, string>, Prism<S, NextParams<T, DisallowTypeChange<string>, true>, string>, Prism<S, NextParams<T, ElemUnion<A>, true>, ElemType<A>>>;
    head(): IfElse<Eq<A, string>, Prism<S, NextParams<T, DisallowTypeChange<string>, true>, string>, Prism<S, NextParams<T, ElemUnion<A>, true>, ElemType<A>>>;
    index(i: number): IfElse<Eq<A, string>, Prism<S, NextParams<T, DisallowTypeChange<string>, true>, string>, Prism<S, NextParams<T, ElemUnion<A>, true>, ElemType<A>>>;
    compose<T2 extends OpticParams, A2>(optic: Traversal<A, T2, A2>): Traversal<S, NextComposeParams<T, T2>, A2>;
    elems(): Traversal<S, NextParams<T, Elems>, ElemType<A>>;
    chars(): RequireString<A, Traversal<S, NextParams<T, DisallowTypeChange<string>>, string>>;
    words(): RequireString<A, Traversal<S, NextParams<T, DisallowTypeChange<string>>, string>>;
    compose<A2>(optic: Getter<A, A2>): Getter<S, A2>;
    to<B>(f: (a: A) => B): Getter<S, B>;
    compose<A2>(optic: AffineFold<A, A2>): AffineFold<S, A2>;
    compose<A2>(optic: Fold<A, A2>): Fold<S, A2>;
    compose<T2 extends OpticParams, A2>(optic: Setter<A, T2, A2>): Setter<S, NextComposeParams<T, T2>, A2>;
    prependTo(): Setter<S, NextParams<T, ElemUnion<A>>, ElemType<A>>;
    appendTo(): Setter<S, NextParams<T, ElemUnion<A>>, ElemType<A>>;
}
export interface Prism<S, T extends OpticParams, A> {
    readonly _tag: 'Prism';
    readonly _removable: T['_R'];
    compose<T2 extends OpticParams, A2>(optic: Equivalence<A, T2, A2>): Prism<S, NextComposeParams<T, T2>, A2>;
    compose<T2 extends OpticParams, A2>(optic: Iso<A, T2, A2>): Prism<S, NextComposeParams<T, T2>, A2>;
    iso<U>(there: (a: A) => U, back: (u: U) => A): Prism<S, NextParams<T, Adapt<A, U>>, U>;
    compose<T2 extends OpticParams, A2>(optic: Lens<A, T2, A2>): Prism<S, NextComposeParams<T, T2>, A2>;
    prop<K extends keyof A>(key: K): Prism<S, NextParams<T, Prop<A, K>>, A[K]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2], K4 extends keyof A[K1][K2][K3], K5 extends keyof A[K1][K2][K3][K4]>(path: [K1, K2, K3, K4, K5]): Prism<S, NextParams<T, Path5<A, K1, K2, K3, K4, K5>>, A[K1][K2][K3][K4][K5]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2], K4 extends keyof A[K1][K2][K3]>(path: [K1, K2, K3, K4]): Prism<S, NextParams<T, Path4<A, K1, K2, K3, K4>>, A[K1][K2][K3][K4]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2]>(path: [K1, K2, K3]): Prism<S, NextParams<T, Path3<A, K1, K2, K3>>, A[K1][K2][K3]>;
    path<K1 extends keyof A, K2 extends keyof A[K1]>(path: [K1, K2]): Prism<S, NextParams<T, Path2<A, K1, K2>>, A[K1][K2]>;
    path<K1 extends keyof A>(path: [K1]): Prism<S, NextParams<T, Prop<A, K1>>, A[K1]>;
    pick<K extends keyof A>(keys: K[]): Prism<S, NextParams<T, Plant<A, K>>, Pick<A, K>>;
    filter(predicate: (item: ElemType<A>) => boolean): Prism<S, NextParams<T, Union<A>>, A>;
    valueOr<B>(defaultValue: B): Prism<S, NextParams<T, Id>, Exclude<A, undefined> | B>;
    compose<T2 extends OpticParams, A2>(optic: Prism<A, T2, A2>): Prism<S, NextComposeParams<T, T2>, A2>;
    optional(): Prism<S, NextParams<T, Optional>, Exclude<A, undefined>>;
    guard_<F extends HKT>(): <U extends A>(g: (a: A) => a is U) => Prism<S, NextParams<T, F>, U>;
    guard<U extends A>(g: (a: A) => a is U): Prism<S, NextParams<T, Choice<A, U>>, U>;
    find(predicate: (item: ElemType<A>) => boolean): Prism<S, NextParams<T, ElemUnion<A>, true>, ElemType<A>>;
    when(predicate: (item: A) => boolean): Prism<S, NextParams<T, Union<A>>, A>;
    at(i: number): IfElse<Eq<A, string>, Prism<S, NextParams<T, DisallowTypeChange<string>, true>, string>, Prism<S, NextParams<T, ElemUnion<A>, true>, ElemType<A>>>;
    head(): IfElse<Eq<A, string>, Prism<S, NextParams<T, DisallowTypeChange<string>, true>, string>, Prism<S, NextParams<T, ElemUnion<A>, true>, ElemType<A>>>;
    index(i: number): IfElse<Eq<A, string>, Prism<S, NextParams<T, DisallowTypeChange<string>, true>, string>, Prism<S, NextParams<T, ElemUnion<A>, true>, ElemType<A>>>;
    compose<T2 extends OpticParams, A2>(optic: Traversal<A, T2, A2>): Traversal<S, NextComposeParams<T, T2>, A2>;
    elems(): Traversal<S, NextParams<T, Elems>, ElemType<A>>;
    chars(): RequireString<A, Traversal<S, NextParams<T, DisallowTypeChange<string>>, string>>;
    words(): RequireString<A, Traversal<S, NextParams<T, DisallowTypeChange<string>>, string>>;
    compose<A2>(optic: Getter<A, A2>): AffineFold<S, A2>;
    to<B>(f: (a: A) => B): AffineFold<S, B>;
    compose<A2>(optic: AffineFold<A, A2>): AffineFold<S, A2>;
    compose<A2>(optic: Fold<A, A2>): Fold<S, A2>;
    compose<T2 extends OpticParams, A2>(optic: Setter<A, T2, A2>): Setter<S, NextComposeParams<T, T2>, A2>;
    prependTo(): Setter<S, NextParams<T, ElemUnion<A>>, ElemType<A>>;
    appendTo(): Setter<S, NextParams<T, ElemUnion<A>>, ElemType<A>>;
}
export interface Traversal<S, T extends OpticParams, A> {
    readonly _tag: 'Traversal';
    readonly _removable: T['_R'];
    compose<T2 extends OpticParams, A2>(optic: Equivalence<A, T2, A2>): Traversal<S, NextComposeParams<T, T2>, A2>;
    compose<T2 extends OpticParams, A2>(optic: Iso<A, T2, A2>): Traversal<S, NextComposeParams<T, T2>, A2>;
    iso<U>(there: (a: A) => U, back: (u: U) => A): Traversal<S, NextParams<T, Adapt<A, U>>, U>;
    compose<T2 extends OpticParams, A2>(optic: Lens<A, T2, A2>): Traversal<S, NextComposeParams<T, T2>, A2>;
    prop<K extends keyof A>(key: K): Traversal<S, NextParams<T, Prop<A, K>>, A[K]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2], K4 extends keyof A[K1][K2][K3], K5 extends keyof A[K1][K2][K3][K4]>(path: [K1, K2, K3, K4, K5]): Traversal<S, NextParams<T, Path5<A, K1, K2, K3, K4, K5>>, A[K1][K2][K3][K4][K5]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2], K4 extends keyof A[K1][K2][K3]>(path: [K1, K2, K3, K4]): Traversal<S, NextParams<T, Path4<A, K1, K2, K3, K4>>, A[K1][K2][K3][K4]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2]>(path: [K1, K2, K3]): Traversal<S, NextParams<T, Path3<A, K1, K2, K3>>, A[K1][K2][K3]>;
    path<K1 extends keyof A, K2 extends keyof A[K1]>(path: [K1, K2]): Traversal<S, NextParams<T, Path2<A, K1, K2>>, A[K1][K2]>;
    path<K1 extends keyof A>(path: [K1]): Traversal<S, NextParams<T, Prop<A, K1>>, A[K1]>;
    pick<K extends keyof A>(keys: K[]): Traversal<S, NextParams<T, Plant<A, K>>, Pick<A, K>>;
    filter(predicate: (item: ElemType<A>) => boolean): Traversal<S, NextParams<T, Union<A>>, A>;
    valueOr<B>(defaultValue: B): Traversal<S, NextParams<T, Id>, Exclude<A, undefined> | B>;
    compose<T2 extends OpticParams, A2>(optic: Prism<A, T2, A2>): Traversal<S, NextComposeParams<T, T2>, A2>;
    optional(): Traversal<S, NextParams<T, Optional>, Exclude<A, undefined>>;
    guard_<F extends HKT>(): <U extends A>(g: (a: A) => a is U) => Traversal<S, NextParams<T, F>, U>;
    guard<U extends A>(g: (a: A) => a is U): Traversal<S, NextParams<T, Choice<A, U>>, U>;
    find(predicate: (item: ElemType<A>) => boolean): Traversal<S, NextParams<T, ElemUnion<A>, true>, ElemType<A>>;
    when(predicate: (item: A) => boolean): Traversal<S, NextParams<T, Union<A>>, A>;
    at(i: number): IfElse<Eq<A, string>, Traversal<S, NextParams<T, DisallowTypeChange<string>, true>, string>, Traversal<S, NextParams<T, ElemUnion<A>, true>, ElemType<A>>>;
    head(): IfElse<Eq<A, string>, Traversal<S, NextParams<T, DisallowTypeChange<string>, true>, string>, Traversal<S, NextParams<T, ElemUnion<A>, true>, ElemType<A>>>;
    index(i: number): IfElse<Eq<A, string>, Traversal<S, NextParams<T, DisallowTypeChange<string>, true>, string>, Traversal<S, NextParams<T, ElemUnion<A>, true>, ElemType<A>>>;
    compose<T2 extends OpticParams, A2>(optic: Traversal<A, T2, A2>): Traversal<S, NextComposeParams<T, T2>, A2>;
    elems(): Traversal<S, NextParams<T, Elems>, ElemType<A>>;
    chars(): RequireString<A, Traversal<S, NextParams<T, DisallowTypeChange<string>>, string>>;
    words(): RequireString<A, Traversal<S, NextParams<T, DisallowTypeChange<string>>, string>>;
    compose<A2>(optic: Getter<A, A2>): Fold<S, A2>;
    to<B>(f: (a: A) => B): Fold<S, B>;
    compose<A2>(optic: AffineFold<A, A2>): Fold<S, A2>;
    compose<A2>(optic: Fold<A, A2>): Fold<S, A2>;
    compose<T2 extends OpticParams, A2>(optic: Setter<A, T2, A2>): Setter<S, NextComposeParams<T, T2>, A2>;
    prependTo(): Setter<S, NextParams<T, ElemUnion<A>>, ElemType<A>>;
    appendTo(): Setter<S, NextParams<T, ElemUnion<A>>, ElemType<A>>;
}
export interface Getter<S, A> {
    readonly _tag: 'Getter';
    compose<T2 extends OpticParams, A2>(optic: Equivalence<A, T2, A2>): Getter<S, A2>;
    compose<T2 extends OpticParams, A2>(optic: Iso<A, T2, A2>): Getter<S, A2>;
    iso<U>(there: (a: A) => U, back: (u: U) => A): Getter<S, U>;
    compose<T2 extends OpticParams, A2>(optic: Lens<A, T2, A2>): Getter<S, A2>;
    prop<K extends keyof A>(key: K): Getter<S, A[K]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2], K4 extends keyof A[K1][K2][K3], K5 extends keyof A[K1][K2][K3][K4]>(path: [K1, K2, K3, K4, K5]): Getter<S, A[K1][K2][K3][K4][K5]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2], K4 extends keyof A[K1][K2][K3]>(path: [K1, K2, K3, K4]): Getter<S, A[K1][K2][K3][K4]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2]>(path: [K1, K2, K3]): Getter<S, A[K1][K2][K3]>;
    path<K1 extends keyof A, K2 extends keyof A[K1]>(path: [K1, K2]): Getter<S, A[K1][K2]>;
    path<K1 extends keyof A>(path: [K1]): Getter<S, A[K1]>;
    pick<K extends keyof A>(keys: K[]): Getter<S, Pick<A, K>>;
    filter(predicate: (item: ElemType<A>) => boolean): Getter<S, A>;
    valueOr<B>(defaultValue: B): Getter<S, Exclude<A, undefined> | B>;
    compose<T2 extends OpticParams, A2>(optic: Prism<A, T2, A2>): AffineFold<S, A2>;
    optional(): AffineFold<S, Exclude<A, undefined>>;
    guard_<F extends HKT>(): <U extends A>(g: (a: A) => a is U) => AffineFold<S, U>;
    guard<U extends A>(g: (a: A) => a is U): AffineFold<S, U>;
    find(predicate: (item: ElemType<A>) => boolean): AffineFold<S, ElemType<A>>;
    when(predicate: (item: A) => boolean): AffineFold<S, A>;
    at(i: number): IfElse<Eq<A, string>, AffineFold<S, string>, AffineFold<S, ElemType<A>>>;
    head(): IfElse<Eq<A, string>, AffineFold<S, string>, AffineFold<S, ElemType<A>>>;
    index(i: number): IfElse<Eq<A, string>, AffineFold<S, string>, AffineFold<S, ElemType<A>>>;
    compose<T2 extends OpticParams, A2>(optic: Traversal<A, T2, A2>): Fold<S, A2>;
    elems(): Fold<S, ElemType<A>>;
    chars(): RequireString<A, Fold<S, string>>;
    words(): RequireString<A, Fold<S, string>>;
    compose<A2>(optic: Getter<A, A2>): Getter<S, A2>;
    to<B>(f: (a: A) => B): Getter<S, B>;
    compose<A2>(optic: AffineFold<A, A2>): AffineFold<S, A2>;
    compose<A2>(optic: Fold<A, A2>): Fold<S, A2>;
}
export interface AffineFold<S, A> {
    readonly _tag: 'AffineFold';
    compose<T2 extends OpticParams, A2>(optic: Equivalence<A, T2, A2>): AffineFold<S, A2>;
    compose<T2 extends OpticParams, A2>(optic: Iso<A, T2, A2>): AffineFold<S, A2>;
    iso<U>(there: (a: A) => U, back: (u: U) => A): AffineFold<S, U>;
    compose<T2 extends OpticParams, A2>(optic: Lens<A, T2, A2>): AffineFold<S, A2>;
    prop<K extends keyof A>(key: K): AffineFold<S, A[K]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2], K4 extends keyof A[K1][K2][K3], K5 extends keyof A[K1][K2][K3][K4]>(path: [K1, K2, K3, K4, K5]): AffineFold<S, A[K1][K2][K3][K4][K5]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2], K4 extends keyof A[K1][K2][K3]>(path: [K1, K2, K3, K4]): AffineFold<S, A[K1][K2][K3][K4]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2]>(path: [K1, K2, K3]): AffineFold<S, A[K1][K2][K3]>;
    path<K1 extends keyof A, K2 extends keyof A[K1]>(path: [K1, K2]): AffineFold<S, A[K1][K2]>;
    path<K1 extends keyof A>(path: [K1]): AffineFold<S, A[K1]>;
    pick<K extends keyof A>(keys: K[]): AffineFold<S, Pick<A, K>>;
    filter(predicate: (item: ElemType<A>) => boolean): AffineFold<S, A>;
    valueOr<B>(defaultValue: B): AffineFold<S, Exclude<A, undefined> | B>;
    compose<T2 extends OpticParams, A2>(optic: Prism<A, T2, A2>): AffineFold<S, A2>;
    optional(): AffineFold<S, Exclude<A, undefined>>;
    guard_<F extends HKT>(): <U extends A>(g: (a: A) => a is U) => AffineFold<S, U>;
    guard<U extends A>(g: (a: A) => a is U): AffineFold<S, U>;
    find(predicate: (item: ElemType<A>) => boolean): AffineFold<S, ElemType<A>>;
    when(predicate: (item: A) => boolean): AffineFold<S, A>;
    at(i: number): IfElse<Eq<A, string>, AffineFold<S, string>, AffineFold<S, ElemType<A>>>;
    head(): IfElse<Eq<A, string>, AffineFold<S, string>, AffineFold<S, ElemType<A>>>;
    index(i: number): IfElse<Eq<A, string>, AffineFold<S, string>, AffineFold<S, ElemType<A>>>;
    compose<T2 extends OpticParams, A2>(optic: Traversal<A, T2, A2>): Fold<S, A2>;
    elems(): Fold<S, ElemType<A>>;
    chars(): RequireString<A, Fold<S, string>>;
    words(): RequireString<A, Fold<S, string>>;
    compose<A2>(optic: Getter<A, A2>): AffineFold<S, A2>;
    to<B>(f: (a: A) => B): AffineFold<S, B>;
    compose<A2>(optic: AffineFold<A, A2>): AffineFold<S, A2>;
    compose<A2>(optic: Fold<A, A2>): Fold<S, A2>;
}
export interface Fold<S, A> {
    readonly _tag: 'Fold';
    compose<T2 extends OpticParams, A2>(optic: Equivalence<A, T2, A2>): Fold<S, A2>;
    compose<T2 extends OpticParams, A2>(optic: Iso<A, T2, A2>): Fold<S, A2>;
    iso<U>(there: (a: A) => U, back: (u: U) => A): Fold<S, U>;
    compose<T2 extends OpticParams, A2>(optic: Lens<A, T2, A2>): Fold<S, A2>;
    prop<K extends keyof A>(key: K): Fold<S, A[K]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2], K4 extends keyof A[K1][K2][K3], K5 extends keyof A[K1][K2][K3][K4]>(path: [K1, K2, K3, K4, K5]): Fold<S, A[K1][K2][K3][K4][K5]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2], K4 extends keyof A[K1][K2][K3]>(path: [K1, K2, K3, K4]): Fold<S, A[K1][K2][K3][K4]>;
    path<K1 extends keyof A, K2 extends keyof A[K1], K3 extends keyof A[K1][K2]>(path: [K1, K2, K3]): Fold<S, A[K1][K2][K3]>;
    path<K1 extends keyof A, K2 extends keyof A[K1]>(path: [K1, K2]): Fold<S, A[K1][K2]>;
    path<K1 extends keyof A>(path: [K1]): Fold<S, A[K1]>;
    pick<K extends keyof A>(keys: K[]): Fold<S, Pick<A, K>>;
    filter(predicate: (item: ElemType<A>) => boolean): Fold<S, A>;
    valueOr<B>(defaultValue: B): Fold<S, Exclude<A, undefined> | B>;
    compose<T2 extends OpticParams, A2>(optic: Prism<A, T2, A2>): Fold<S, A2>;
    optional(): Fold<S, Exclude<A, undefined>>;
    guard_<F extends HKT>(): <U extends A>(g: (a: A) => a is U) => Fold<S, U>;
    guard<U extends A>(g: (a: A) => a is U): Fold<S, U>;
    find(predicate: (item: ElemType<A>) => boolean): Fold<S, ElemType<A>>;
    when(predicate: (item: A) => boolean): Fold<S, A>;
    at(i: number): IfElse<Eq<A, string>, Fold<S, string>, Fold<S, ElemType<A>>>;
    head(): IfElse<Eq<A, string>, Fold<S, string>, Fold<S, ElemType<A>>>;
    index(i: number): IfElse<Eq<A, string>, Fold<S, string>, Fold<S, ElemType<A>>>;
    compose<T2 extends OpticParams, A2>(optic: Traversal<A, T2, A2>): Fold<S, A2>;
    elems(): Fold<S, ElemType<A>>;
    chars(): RequireString<A, Fold<S, string>>;
    words(): RequireString<A, Fold<S, string>>;
    compose<A2>(optic: Getter<A, A2>): Fold<S, A2>;
    to<B>(f: (a: A) => B): Fold<S, B>;
    compose<A2>(optic: AffineFold<A, A2>): Fold<S, A2>;
    compose<A2>(optic: Fold<A, A2>): Fold<S, A2>;
}
export interface Setter<S, T extends OpticParams, A> {
    readonly _tag: 'Setter';
    readonly _removable: T['_R'];
}
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Equivalence<S, T, A>, optic2: Equivalence<A, T2, A2>): Equivalence<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Equivalence<S, T, A>, optic2: Iso<A, T2, A2>): Iso<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Equivalence<S, T, A>, optic2: Lens<A, T2, A2>): Lens<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Equivalence<S, T, A>, optic2: Prism<A, T2, A2>): Prism<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Equivalence<S, T, A>, optic2: Traversal<A, T2, A2>): Traversal<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, A2>(optic1: Equivalence<S, T, A>, optic2: Getter<A, A2>): Getter<S, A2>;
export declare function compose<S, T extends OpticParams, A, A2>(optic1: Equivalence<S, T, A>, optic2: AffineFold<A, A2>): AffineFold<S, A2>;
export declare function compose<S, T extends OpticParams, A, A2>(optic1: Equivalence<S, T, A>, optic2: Fold<A, A2>): Fold<S, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Equivalence<S, T, A>, optic2: Setter<A, T2, A2>): Setter<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Iso<S, T, A>, optic2: Equivalence<A, T2, A2>): Iso<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Iso<S, T, A>, optic2: Iso<A, T2, A2>): Iso<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Iso<S, T, A>, optic2: Lens<A, T2, A2>): Lens<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Iso<S, T, A>, optic2: Prism<A, T2, A2>): Prism<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Iso<S, T, A>, optic2: Traversal<A, T2, A2>): Traversal<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, A2>(optic1: Iso<S, T, A>, optic2: Getter<A, A2>): Getter<S, A2>;
export declare function compose<S, T extends OpticParams, A, A2>(optic1: Iso<S, T, A>, optic2: AffineFold<A, A2>): AffineFold<S, A2>;
export declare function compose<S, T extends OpticParams, A, A2>(optic1: Iso<S, T, A>, optic2: Fold<A, A2>): Fold<S, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Iso<S, T, A>, optic2: Setter<A, T2, A2>): Setter<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Lens<S, T, A>, optic2: Equivalence<A, T2, A2>): Lens<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Lens<S, T, A>, optic2: Iso<A, T2, A2>): Lens<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Lens<S, T, A>, optic2: Lens<A, T2, A2>): Lens<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Lens<S, T, A>, optic2: Prism<A, T2, A2>): Prism<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Lens<S, T, A>, optic2: Traversal<A, T2, A2>): Traversal<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, A2>(optic1: Lens<S, T, A>, optic2: Getter<A, A2>): Getter<S, A2>;
export declare function compose<S, T extends OpticParams, A, A2>(optic1: Lens<S, T, A>, optic2: AffineFold<A, A2>): AffineFold<S, A2>;
export declare function compose<S, T extends OpticParams, A, A2>(optic1: Lens<S, T, A>, optic2: Fold<A, A2>): Fold<S, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Lens<S, T, A>, optic2: Setter<A, T2, A2>): Setter<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Prism<S, T, A>, optic2: Equivalence<A, T2, A2>): Prism<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Prism<S, T, A>, optic2: Iso<A, T2, A2>): Prism<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Prism<S, T, A>, optic2: Lens<A, T2, A2>): Prism<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Prism<S, T, A>, optic2: Prism<A, T2, A2>): Prism<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Prism<S, T, A>, optic2: Traversal<A, T2, A2>): Traversal<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, A2>(optic1: Prism<S, T, A>, optic2: Getter<A, A2>): AffineFold<S, A2>;
export declare function compose<S, T extends OpticParams, A, A2>(optic1: Prism<S, T, A>, optic2: AffineFold<A, A2>): AffineFold<S, A2>;
export declare function compose<S, T extends OpticParams, A, A2>(optic1: Prism<S, T, A>, optic2: Fold<A, A2>): Fold<S, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Prism<S, T, A>, optic2: Setter<A, T2, A2>): Setter<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Traversal<S, T, A>, optic2: Equivalence<A, T2, A2>): Traversal<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Traversal<S, T, A>, optic2: Iso<A, T2, A2>): Traversal<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Traversal<S, T, A>, optic2: Lens<A, T2, A2>): Traversal<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Traversal<S, T, A>, optic2: Prism<A, T2, A2>): Traversal<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Traversal<S, T, A>, optic2: Traversal<A, T2, A2>): Traversal<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, T extends OpticParams, A, A2>(optic1: Traversal<S, T, A>, optic2: Getter<A, A2>): Fold<S, A2>;
export declare function compose<S, T extends OpticParams, A, A2>(optic1: Traversal<S, T, A>, optic2: AffineFold<A, A2>): Fold<S, A2>;
export declare function compose<S, T extends OpticParams, A, A2>(optic1: Traversal<S, T, A>, optic2: Fold<A, A2>): Fold<S, A2>;
export declare function compose<S, T extends OpticParams, A, T2 extends OpticParams, A2>(optic1: Traversal<S, T, A>, optic2: Setter<A, T2, A2>): Setter<S, NextComposeParams<T, T2>, A2>;
export declare function compose<S, A, T2 extends OpticParams, A2>(optic1: Getter<S, A>, optic2: Equivalence<A, T2, A2>): Getter<S, A2>;
export declare function compose<S, A, T2 extends OpticParams, A2>(optic1: Getter<S, A>, optic2: Iso<A, T2, A2>): Getter<S, A2>;
export declare function compose<S, A, T2 extends OpticParams, A2>(optic1: Getter<S, A>, optic2: Lens<A, T2, A2>): Getter<S, A2>;
export declare function compose<S, A, T2 extends OpticParams, A2>(optic1: Getter<S, A>, optic2: Prism<A, T2, A2>): AffineFold<S, A2>;
export declare function compose<S, A, T2 extends OpticParams, A2>(optic1: Getter<S, A>, optic2: Traversal<A, T2, A2>): Fold<S, A2>;
export declare function compose<S, A, A2>(optic1: Getter<S, A>, optic2: Getter<A, A2>): Getter<S, A2>;
export declare function compose<S, A, A2>(optic1: Getter<S, A>, optic2: AffineFold<A, A2>): AffineFold<S, A2>;
export declare function compose<S, A, A2>(optic1: Getter<S, A>, optic2: Fold<A, A2>): Fold<S, A2>;
export declare function compose<S, A, T2 extends OpticParams, A2>(optic1: AffineFold<S, A>, optic2: Equivalence<A, T2, A2>): AffineFold<S, A2>;
export declare function compose<S, A, T2 extends OpticParams, A2>(optic1: AffineFold<S, A>, optic2: Iso<A, T2, A2>): AffineFold<S, A2>;
export declare function compose<S, A, T2 extends OpticParams, A2>(optic1: AffineFold<S, A>, optic2: Lens<A, T2, A2>): AffineFold<S, A2>;
export declare function compose<S, A, T2 extends OpticParams, A2>(optic1: AffineFold<S, A>, optic2: Prism<A, T2, A2>): AffineFold<S, A2>;
export declare function compose<S, A, T2 extends OpticParams, A2>(optic1: AffineFold<S, A>, optic2: Traversal<A, T2, A2>): Fold<S, A2>;
export declare function compose<S, A, A2>(optic1: AffineFold<S, A>, optic2: Getter<A, A2>): AffineFold<S, A2>;
export declare function compose<S, A, A2>(optic1: AffineFold<S, A>, optic2: AffineFold<A, A2>): AffineFold<S, A2>;
export declare function compose<S, A, A2>(optic1: AffineFold<S, A>, optic2: Fold<A, A2>): Fold<S, A2>;
export declare function compose<S, A, T2 extends OpticParams, A2>(optic1: Fold<S, A>, optic2: Equivalence<A, T2, A2>): Fold<S, A2>;
export declare function compose<S, A, T2 extends OpticParams, A2>(optic1: Fold<S, A>, optic2: Iso<A, T2, A2>): Fold<S, A2>;
export declare function compose<S, A, T2 extends OpticParams, A2>(optic1: Fold<S, A>, optic2: Lens<A, T2, A2>): Fold<S, A2>;
export declare function compose<S, A, T2 extends OpticParams, A2>(optic1: Fold<S, A>, optic2: Prism<A, T2, A2>): Fold<S, A2>;
export declare function compose<S, A, T2 extends OpticParams, A2>(optic1: Fold<S, A>, optic2: Traversal<A, T2, A2>): Fold<S, A2>;
export declare function compose<S, A, A2>(optic1: Fold<S, A>, optic2: Getter<A, A2>): Fold<S, A2>;
export declare function compose<S, A, A2>(optic1: Fold<S, A>, optic2: AffineFold<A, A2>): Fold<S, A2>;
export declare function compose<S, A, A2>(optic1: Fold<S, A>, optic2: Fold<A, A2>): Fold<S, A2>;
export declare function optic<S>(): OpticFor<S>;
export declare function optic_<S>(): OpticFor_<S>;
export declare function get<S, A>(optic: Equivalence<S, any, A> | Iso<S, any, A> | Lens<S, any, A> | Getter<S, A>): (source: S) => A;
export declare function preview<S, A>(optic: Prism<S, any, A> | Traversal<S, any, A> | AffineFold<S, A> | Fold<S, A>): (source: S) => A | undefined;
export declare function collect<S, A>(optic: Prism<S, any, A> | Traversal<S, any, A> | Fold<S, A>): (source: S) => A[];
export declare function modify<S, T extends OpticParams, A>(optic: Equivalence<S, T, A> | Iso<S, T, A> | Lens<S, T, A> | Prism<S, T, A> | Traversal<S, T, A>): <B>(f: (a: A) => B) => (source: S) => Simplify<S, Apply<T['_T'], B>>;
export declare function set<S, T extends OpticParams, A>(optic: Equivalence<S, T, A> | Iso<S, T, A> | Lens<S, T, A> | Prism<S, T, A> | Traversal<S, T, A> | Setter<S, T, A>): <B>(value: B) => (source: S) => Simplify<S, Apply<T['_T'], B>>;
export declare function remove<S>(optic: Prism<S, Params<any, true>, any> | Traversal<S, Params<any, true>, any>): (source: S) => S;
export declare function pipe<A>(a: A): A;
export declare function pipe<A, B>(a: A, ab: (a: A) => B): B;
export declare function pipe<A, B, C>(a: A, ab: (a: A) => B, bc: (b: B) => C): C;
export declare function pipe<A, B, C, D>(a: A, ab: (a: A) => B, bc: (b: B) => C, cd: (c: C) => D): D;
export declare function pipe<A, B, C, D, E>(a: A, ab: (a: A) => B, bc: (b: B) => C, cd: (c: C) => D, de: (d: D) => E): E;
export declare function pipe<A, B, C, D, E, F>(a: A, ab: (a: A) => B, bc: (b: B) => C, cd: (c: C) => D, de: (d: D) => E, ef: (e: E) => F): F;
export declare function pipe<A, B, C, D, E, F, G>(a: A, ab: (a: A) => B, bc: (b: B) => C, cd: (c: C) => D, de: (d: D) => E, ef: (e: E) => F, fg: (f: F) => G): G;
export declare function pipe<A, B, C, D, E, F, G, H>(a: A, ab: (a: A) => B, bc: (b: B) => C, cd: (c: C) => D, de: (d: D) => E, ef: (e: E) => F, fg: (f: F) => G, gh: (g: G) => H): H;
export declare function pipe<A, B, C, D, E, F, G, H, I>(a: A, ab: (a: A) => B, bc: (b: B) => C, cd: (c: C) => D, de: (d: D) => E, ef: (e: E) => F, fg: (f: F) => G, gh: (g: G) => H, hi: (h: H) => I): I;
export declare function pipe<A, B, C, D, E, F, G, H, I, J>(a: A, ab: (a: A) => B, bc: (b: B) => C, cd: (c: C) => D, de: (d: D) => E, ef: (e: E) => F, fg: (f: F) => G, gh: (g: G) => H, hi: (h: H) => I, ij: (i: I) => J): J;
